﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.JSInterop;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Authorization;
using Microsoft.AspNetCore.Components.Web;
using Radzen;
using Radzen.Blazor;
using SasInfoSys.Models.SasFresh;
using SasInfoSys.Models.SasService;
using Microsoft.EntityFrameworkCore;
using Microsoft.AspNetCore.Identity;
using SasInfoSys.Models;

namespace SasInfoSys.Pages
{
    public partial class OrderMaintenanceComponent : ComponentBase
    {
        [Parameter(CaptureUnmatchedValues = true)]
        public IReadOnlyDictionary<string, dynamic> Attributes { get; set; }

        public void Reload()
        {
            InvokeAsync(StateHasChanged);
        }

        public void OnPropertyChanged(PropertyChangedEventArgs args)
        {
        }

        [Inject]
        protected IJSRuntime JSRuntime { get; set; }

        [Inject]
        protected NavigationManager UriHelper { get; set; }

        [Inject]
        protected DialogService DialogService { get; set; }

        [Inject]
        protected TooltipService TooltipService { get; set; }

        [Inject]
        protected ContextMenuService ContextMenuService { get; set; }

        [Inject]
        protected NotificationService NotificationService { get; set; }

        [Inject]
        protected SecurityService Security { get; set; }

        [Inject]
        protected AuthenticationStateProvider AuthenticationStateProvider { get; set; }

        [Inject]
        protected SasFreshService SasFresh { get; set; }

        [Inject]
        protected SasServiceService SasService { get; set; }
        protected RadzenDataGrid<SasInfoSys.Models.SasFresh.VwInfOrder> grid0;
        protected RadzenDataGrid<SasInfoSys.Models.SasFresh.InfOrderItemProduct> datagrid0;
        protected RadzenDataGrid<SasInfoSys.Models.SasFresh.InfPoOiProduct> datagrid2;
        protected RadzenDataGrid<SasInfoSys.Models.SasFresh.InfOrderItemFabric> datagrid1;

        string _OrderRefNo;
        protected string OrderRefNo
        {
            get
            {
                return _OrderRefNo;
            }
            set
            {
                if (!object.Equals(_OrderRefNo, value))
                {
                    var args = new PropertyChangedEventArgs(){ Name = "OrderRefNo", NewValue = value, OldValue = _OrderRefNo };
                    _OrderRefNo = value;
                    OnPropertyChanged(args);
                    Reload();
                }
            }
        }

        IEnumerable<SasInfoSys.Models.SasFresh.VwInfOrder> _getVwInfOrdersResult;
        protected IEnumerable<SasInfoSys.Models.SasFresh.VwInfOrder> getVwInfOrdersResult
        {
            get
            {
                return _getVwInfOrdersResult;
            }
            set
            {
                if (!object.Equals(_getVwInfOrdersResult, value))
                {
                    var args = new PropertyChangedEventArgs(){ Name = "getVwInfOrdersResult", NewValue = value, OldValue = _getVwInfOrdersResult };
                    _getVwInfOrdersResult = value;
                    OnPropertyChanged(args);
                    Reload();
                }
            }
        }

        bool _HaveError;
        protected bool HaveError
        {
            get
            {
                return _HaveError;
            }
            set
            {
                if (!object.Equals(_HaveError, value))
                {
                    var args = new PropertyChangedEventArgs(){ Name = "HaveError", NewValue = value, OldValue = _HaveError };
                    _HaveError = value;
                    OnPropertyChanged(args);
                    Reload();
                }
            }
        }

        SasInfoSys.Models.SasFresh.VwInfOrder _vwInfOrder;
        protected SasInfoSys.Models.SasFresh.VwInfOrder vwInfOrder
        {
            get
            {
                return _vwInfOrder;
            }
            set
            {
                if (!object.Equals(_vwInfOrder, value))
                {
                    var args = new PropertyChangedEventArgs(){ Name = "vwInfOrder", NewValue = value, OldValue = _vwInfOrder };
                    _vwInfOrder = value;
                    OnPropertyChanged(args);
                    Reload();
                }
            }
        }

        IEnumerable<SasInfoSys.Models.SasFresh.InfOrderItemProduct> _Prods;
        protected IEnumerable<SasInfoSys.Models.SasFresh.InfOrderItemProduct> Prods
        {
            get
            {
                return _Prods;
            }
            set
            {
                if (!object.Equals(_Prods, value))
                {
                    var args = new PropertyChangedEventArgs(){ Name = "Prods", NewValue = value, OldValue = _Prods };
                    _Prods = value;
                    OnPropertyChanged(args);
                    Reload();
                }
            }
        }

        protected override async System.Threading.Tasks.Task OnInitializedAsync()
        {
            await Security.InitializeAsync(AuthenticationStateProvider);
            if (!Security.IsAuthenticated())
            {
                UriHelper.NavigateTo("Login", true);
            }
            else
            {
                await Load();
            }
        }
        protected async System.Threading.Tasks.Task Load()
        {
            OrderRefNo = OrderRefNo;

            try
            {
                var sasFreshGetVwInfOrdersResult = await SasFresh.GetVwInfOrders(new Query() { Filter = $@"i => i.OrderNo != @0", FilterParameters = new object[] { "Not Allocated" }, OrderBy = $"OrderCreatedDate desc" });
                getVwInfOrdersResult = sasFreshGetVwInfOrdersResult;
            }
            catch (System.Exception sasFreshGetVwInfOrdersException)
            {
                NotificationService.Notify(new NotificationMessage(){ Severity = NotificationSeverity.Info,Summary = $"No orders found",Detail = $"{sasFreshGetVwInfOrdersException.Message}" });
            }
        }

        protected async System.Threading.Tasks.Task Splitbutton0Click(RadzenSplitButtonItem args)
        {
            if (args?.Value == "csv")
            {
                await SasFresh.ExportVwInfOrdersToCSV(new Query() { Filter = $@"{grid0.Query.Filter}", OrderBy = $"{grid0.Query.OrderBy}", Expand = "", Select = "FirstName,LastName,Email,SalesPerson,SalesPerson2,OrderType,OrderNo,OrderCreatedDate,Postcode,OrderTotal" }, $"Order Maintenance");

            }

            if (args == null || args.Value == "xlsx")
            {
                await SasFresh.ExportVwInfOrdersToExcel(new Query() { Filter = $@"{grid0.Query.Filter}", OrderBy = $"{grid0.Query.OrderBy}", Expand = "", Select = "FirstName,LastName,Email,SalesPerson,SalesPerson2,OrderType,OrderNo,OrderCreatedDate,Postcode,OrderTotal" }, $"Order Maintenance");

            }
        }

        protected async void Grid0CellRender(DataGridCellRenderEventArgs<SasInfoSys.Models.SasFresh.VwInfOrder> args)
        {
            if (args.Column.Property == "IsConfirmed" && args.Data.OrderType != "Quote" && args.Data.OrderCreateBy != "oldmis") {
args.Attributes.Add("style", $"background-color: {((bool)args.Data.IsConfirmed ? "White":"Red")};");
};
        }

        protected async System.Threading.Tasks.Task Grid0RowSelect(SasInfoSys.Models.SasFresh.VwInfOrder args)
        {
            HaveError = false;

            vwInfOrder = args;

            try
            {
                var getOrderProductsResult = await GetOrderProducts(vwInfOrder.OrderId);
                Prods = getOrderProductsResult;
            }
            catch (System.Exception getOrderProductsException)
            {
                NotificationService.Notify(new NotificationMessage(){ Severity = NotificationSeverity.Error,Summary = $"Unable to get order details",Detail = $"{getOrderProductsException.Message}" });
            }
        }

        protected async System.Threading.Tasks.Task Button0Click(MouseEventArgs args, dynamic data)
        {
                NotificationService.Notify(new NotificationMessage(){ Severity = NotificationSeverity.Info,Summary = $"Feature not implemented." });
        }
    }
}
